      subroutine rate(ijx,nx,ny,jpoly,
     1     lsp,cp,sshf,ewss,nsss,ssr,
     2     lsp9,cp9,sshf9,ewss9,nsss9,ssr9,
     3     lsp6,cp6,sshf6,ewss6,nsss6,ssr6)

********************************************************************
*                                                                  *
*                     Author: P. JAMES                             *
*                     2000-12-11                                   *
*                     adapted: A. BECK                             *
*                     2003-06-01                                   *
*                                                                  *
********************************************************************
*                                                                  *
* DESCRIPTION: This program converts a set of accumulated fluxes   *
* ECMWF model FC fields into equivalent RATES at time ijx          *
* LSP,CP    [mm/hr]                                                *
* SSR,SSHF  [W/(m*m)]                                              *
* EWSS,NSSS [N/(m*m)]                                              *
* This version for fields after 12-09-2000 in which only FC fields *
* with steps of 3, 6, 9 and 12 hours are available due to ECMWFs   *
* 4D-var assimilation system    (ie. no FG fields)                 *
*                                                                  *
********************************************************************

      integer ijx,ijp4,ijpm,i,j,iact
      real step,steprain

      real lsp(nx,ny,jpoly),cp(nx,ny,jpoly),sshf(nx,ny,jpoly),
     &     ewss(nx,ny,jpoly),nsss(nx,ny,jpoly),
     &     ssr(nx,ny,jpoly)
      real lsp6(nx,ny),cp6(nx,ny),sshf6(nx,ny),
     &     ewss6(nx,ny),nsss6(nx,ny),ssr6(nx,ny)
      real lsp9(nx,ny),cp9(nx,ny),sshf9(nx,ny),
     &     ewss9(nx,ny),nsss9(nx,ny),ssr9(nx,ny)

      NAMELIST /NAMFX2/
     1            NX, NY, 
     2            MAXTIME,JPOLY,
     3            JHRF,
     4            RLO0, RLA0, DX, DY

      call posnam (4,'NAMFX2')
      read (4,NAMFX2)

      if (jhrf.eq.3) then
         step=3600.*3.
         steprain=3./1000.
      else
         step=3600.*6.
         steprain=6./1000.
      end if

      ijp4=mod(ijx,4)
      ijpm=mod(ijx,2)
      iact=ijx
      if (iact.gt.4) iact=4

      print*, jhrf, ijp4, ijpm, iact

      if (jhrf.eq.3) then
* For 3-hourly data
      if (ijp4.eq.1) then
* Fluxes with STEP=03, no change needed
      do 75 i=1,nx
      do 75 j=1,ny
      lsp(i,j,iact)=lsp(i,j,iact)/steprain
      cp(i,j,iact)=cp(i,j,iact)/steprain
      sshf(i,j,iact)=sshf(i,j,iact)/step
      ssr(i,j,iact)=ssr(i,j,iact)/step
      ewss(i,j,iact)=ewss(i,j,iact)/step
      nsss(i,j,iact)=nsss(i,j,iact)/step
75    continue
      else if (ijp4.eq.2) then
* Fluxes with STEP=06
* First of all, store the normalised STEP=06 values
      do 24 i=1,nx
      do 24 j=1,ny
      lsp6(i,j)=lsp(i,j,iact)/steprain
      cp6(i,j)=cp(i,j,iact)/steprain
      sshf6(i,j)=sshf(i,j,iact)/step
      ssr6(i,j)=ssr(i,j,iact)/step
      ewss6(i,j)=ewss(i,j,iact)/step
      nsss6(i,j)=nsss(i,j,iact)/step
24    continue
* Then subtract previous 3-hourly values 
      do 25 i=1,nx
      do 25 j=1,ny
      lsp(i,j,iact)=lsp(i,j,iact)/steprain-lsp(i,j,iact-1)
      cp(i,j,iact)=cp(i,j,iact)/steprain-cp(i,j,iact-1)
      sshf(i,j,iact)=sshf(i,j,iact)/step-sshf(i,j,iact-1)
      ssr(i,j,iact)=ssr(i,j,iact)/step-ssr(i,j,iact-1)
      ewss(i,j,iact)=ewss(i,j,iact)/step-ewss(i,j,iact-1)
      nsss(i,j,iact)=nsss(i,j,iact)/step-nsss(i,j,iact-1)
25    continue
      else if (ijp4.eq.3) then
* Fluxes with STEP=09
* First of all, store the normalised STEP=09 values
      do 34 i=1,nx
      do 34 j=1,ny
      lsp9(i,j)=lsp(i,j,iact)/steprain
      cp9(i,j)=cp(i,j,iact)/steprain
      sshf9(i,j)=sshf(i,j,iact)/step
      ssr9(i,j)=ssr(i,j,iact)/step
      ewss9(i,j)=ewss(i,j,iact)/step
      nsss9(i,j)=nsss(i,j,iact)/step
34    continue
* Then subtract previous STEP=06 values 
      do 35 i=1,nx
      do 35 j=1,ny
      lsp(i,j,iact)=lsp(i,j,iact)/steprain-lsp6(i,j)
      cp(i,j,iact)=cp(i,j,iact)/steprain-cp6(i,j)
      sshf(i,j,iact)=sshf(i,j,iact)/step-sshf6(i,j)
      ssr(i,j,iact)=ssr(i,j,iact)/step-ssr6(i,j)
      ewss(i,j,iact)=ewss(i,j,iact)/step-ewss6(i,j)
      nsss(i,j,iact)=nsss(i,j,iact)/step-nsss6(i,j)
35    continue
      else
* Fluxes with STEP=12
* Subtract previous STEP=09 values 
      do 45 i=1,nx
      do 45 j=1,ny
      lsp(i,j,iact)=lsp(i,j,iact)/steprain-lsp9(i,j)
      cp(i,j,iact)=cp(i,j,iact)/steprain-cp9(i,j)
      sshf(i,j,iact)=sshf(i,j,iact)/step-sshf9(i,j)
      ssr(i,j,iact)=ssr(i,j,iact)/step-ssr9(i,j)
      ewss(i,j,iact)=ewss(i,j,iact)/step-ewss9(i,j)
      nsss(i,j,iact)=nsss(i,j,iact)/step-nsss9(i,j)
45    continue
      end if
      else
* For 6-hourly data
      if (ijpm.eq.1) then
* Fluxes with STEP=03, no change needed
      do 55 i=1,nx
      do 55 j=1,ny
      lsp(i,j,iact)=lsp(i,j,iact)/steprain
      cp(i,j,iact)=cp(i,j,iact)/steprain
      sshf(i,j,iact)=sshf(i,j,iact)/step
      ssr(i,j,iact)=ssr(i,j,iact)/step
      ewss(i,j,iact)=ewss(i,j,iact)/step
      nsss(i,j,iact)=nsss(i,j,iact)/step
55    continue
      else
* Fluxes with STEP=06
* Subtract previous 3-hourly values 
      do 65 i=1,nx
      do 65 j=1,ny
      lsp(i,j,iact)=lsp(i,j,iact)/steprain-lsp(i,j,iact-1)
      cp(i,j,iact)=cp(i,j,iact)/steprain-cp(i,j,iact-1)
      sshf(i,j,iact)=sshf(i,j,iact)/step-sshf(i,j,iact-1)
      ssr(i,j,iact)=ssr(i,j,iact)/step-ssr(i,j,iact-1)
      ewss(i,j,iact)=ewss(i,j,iact)/step-ewss(i,j,iact-1)
      nsss(i,j,iact)=nsss(i,j,iact)/step-nsss(i,j,iact-1)
65    continue
      end if
      end if
      return
      end
