      subroutine readflux
********************************************************************
*                                                                  *
*                     Author: P. JAMES                             *
*                     2000-11-12                                   *
*                     adapted: A. BECK                             *
*                     2003-06-01                                   *
*                                                                  *
********************************************************************
*                                                                  *
* DESCRIPTION: This program reads accumulated values of            *
*                                                                  *
*     QUANTITY                     MARS ABBR    CODE               *
*                                                                  *
*     large scale precipitation       LSP       142                *
*     convective precipitation        CP        143                *
*     surface sensible heat flux      SSHF      146                *
*     surface solar radiation         SSR       176                *
*     u stress                        EWSS      180                *
*     v stress                        NSSS      181                * 
*                                                                  *
*                                                                  *
********************************************************************

      integer kunit1,kunit2,ierr,lenght,iword,i,j,itime
      integer idnum(12),itges,ijp,kunit3,kunit4
      integer ij,im,id,ih,idel,ind,ic,jmd,hms,jout
      integer ISEC0(2),ISEC1(56)
      integer ISEC3(2),ISEC4(42)
      real ZSEC2(122),ZSEC3(2)

      character*10 fname1,fname2,fname3,fname4
      character*1 mode

      double precision jul,juldate

      integer, allocatable, dimension(:) :: inbuff, isec2
      real, allocatable, dimension(:) :: zsec4

      real, allocatable, dimension(:,:,:) :: lsp,cp,sshf,ewss,nsss,ssr
      real, allocatable, dimension(:,:) :: lsp6,cp6,sshf6,ewss6,
     1     nsss6,ssr6
      real, allocatable, dimension(:,:) :: lsp9,cp9,sshf9,ewss9,
     1     nsss9,ssr9
      real, allocatable, dimension(:,:) :: lspi,cpi,sshfi,ewssi,
     1     nsssi,ssri
      integer,allocatable, dimension(:) :: idate

C
C Change directory info below, character length must
C be consistent with that above for fname1 etc.
C

      data fname1 /'surf_03_ub'/
      data fname2 /'surf_06_ub'/
      data fname3 /'surf_09_ub'/
      data fname4 /'surf_12_ub'/
      data idnum /31,0,31,30,31,30,31,31,30,31,30,31/
*
      NAMELIST /NAMFX2/
     1            NX, NY, 
     2            MAXTIME,JPOLY,
     3            JHRF,
     4            RLO0, RLA0, DX, DY

      call posnam (4,'NAMFX2')
      read (4,NAMFX2)

      idimens=nx*ny

      allocate ( INBUFF(IDIMENS*4), ISEC2(22+nx+ny) )
      allocate ( ZSEC4(IDIMENS*16) )

      allocate(lsp(nx,ny,jpoly),cp(nx,ny,jpoly),sshf(nx,ny,jpoly),
     &     ewss(nx,ny,jpoly),nsss(nx,ny,jpoly),
     &     ssr(nx,ny,jpoly))

      allocate(lsp6(nx,ny),cp6(nx,ny),sshf6(nx,ny),
     &     ewss6(nx,ny),nsss6(nx,ny),ssr6(nx,ny))

      allocate(lsp9(nx,ny),cp9(nx,ny),sshf9(nx,ny),
     &     ewss9(nx,ny),nsss9(nx,ny),ssr9(nx,ny))

      allocate(lspi(nx,ny),cpi(nx,ny),
     &     sshfi(nx,ny),ewssi(nx,ny),nsssi(nx,ny),ssri(nx,ny))

      allocate(idate(maxtime))


      mode='D'
      if (jhrf.eq.3) then
      call PBOPEN(kunit1,fname1,'r',ierr)
      if(ierr.lt.0) stop ' SUBROUTINE PBOPEN: ERROR OPENING FILE'
      end if
      call PBOPEN(kunit2,fname2,'r',ierr)
      if(ierr.lt.0) stop ' SUBROUTINE PBOPEN: ERROR OPENING FILE'
      if (jhrf.eq.3) then
      call PBOPEN(kunit3,fname3,'r',ierr)
      if(ierr.lt.0) stop ' SUBROUTINE PBOPEN: ERROR OPENING FILE'
      end if
      call PBOPEN(kunit4,fname4,'r',ierr)
      if(ierr.lt.0) stop ' SUBROUTINE PBOPEN: ERROR OPENING FILE'
*
      do 90 ijp=1,3
* if jhrf=6, then first 3 times are read from kunit2,4,2
* if jhrf=3, then cycles kunit1,2,3,4...
      do 100 ind=1,6
        if (jhrf.eq.3) then
          if (mod(ijp,4).eq.1) then
        call PBGRIB(kunit1,INBUFF,IDIMENS*8,lenght,ierr)
          else if (mod(ijp,4).eq.2) then 
        call PBGRIB(kunit2,INBUFF,IDIMENS*8,lenght,ierr)
          else
        call PBGRIB(kunit3,INBUFF,IDIMENS*8,lenght,ierr)
          end if
        else
          if (mod(ijp,2).eq.1) then
        call PBGRIB(kunit2,INBUFF,IDIMENS*8,lenght,ierr)
          else
        call PBGRIB(kunit4,INBUFF,IDIMENS*8,lenght,ierr)
          end if
        end if
        if(ierr.eq.-1) stop 'ERROR: EOF DETECTED AT TIME<3'
        if(ierr.lt.-1) stop 'SORRY: GRIB READ ERROR DETECTED'
        if(ierr.ne.0) stop 'SORRY: GRIB READ ERROR DETECTED'
        call GRIBEX (ISEC0,ISEC1,ISEC2,ZSEC2,ISEC3,ZSEC3,ISEC4,
     &               ZSEC4,IDIMENS,INBUFF,IDIMENS*8,iword,mode,ierr)
        jout=ISEC0(1)
        if(ierr.ne.0) stop 'SORRY: GRIB CONVERSION ERROR DETECTED'
        ic=isec1(21)-1
        ij=isec1(10)
        im=isec1(11)
        id=isec1(12)
        ih=isec1(13)
        idel=isec1(16)
        jmd=ic*1000000+ij*10000+im*100+id
        hms=ih*10000
        jul=juldate(jmd,hms)
        jul=jul+dble(idel)/dble(24.0)
        call caldate(jul,jmd,hms)
        idate(ijp)=jmd*100+hms/10000

!        PRINT*, ic,ij,im,id,ih,idel
!        PRINT*, ijp, idate(ijp)

        if(ind.eq.1) write(*,'(a,i10.10)') 'DATE: ',idate(ijp)
        if(mod(ih,3).ne.0) stop 'SORRY: ILLEGAL TIME DETECTED'      
        do 50 i=1,nx
          do 50 j=1,ny
            if(isec1(6).eq.142) lsp (i,j,ijp)=zsec4(nx*(ny-j)+i)
            if(isec1(6).eq.143) cp  (i,j,ijp)=zsec4(nx*(ny-j)+i)
            if(isec1(6).eq.146) sshf(i,j,ijp)=zsec4(nx*(ny-j)+i)
            if(isec1(6).eq.176) ssr (i,j,ijp)=zsec4(nx*(ny-j)+i)
            if(isec1(6).eq.180) ewss(i,j,ijp)=zsec4(nx*(ny-j)+i)
            if(isec1(6).eq.181) nsss(i,j,ijp)=zsec4(nx*(ny-j)+i)
50      continue
        if(isec1(6).eq.142) then
          write(*,'(6x,a,4x,i3)') 
     &      'FINISHED: LSP ',isec1(6)
        endif
        if(isec1(6).eq.143) then
          write(*,'(6x,a,4x,i3)') 
     &      'FINISHED: CP  ',isec1(6)
        endif
        if(isec1(6).eq.146) then
          write(*,'(6x,a,4x,i3)') 
     &      'FINISHED: SSHF',isec1(6)
        endif
        if(isec1(6).eq.176) then
          write(*,'(6x,a,4x,i3)') 
     &      'FINISHED: SSR ',isec1(6)
        endif
        if(isec1(6).eq.180) then
          write(*,'(6x,a,4x,i3)') 
     &      'FINISHED: EWSS',isec1(6)
        endif
        if(isec1(6).eq.181) then
          write(*,'(6x,a,4x,i3)') 
     &      'FINISHED: NSSS',isec1(6)
        endif
100   continue
*
* Convert fluxes to consistent 3- or 6- hourly RATEs
      call rate(ijp,nx,ny,jpoly,
     1     lsp,cp,sshf,ewss,nsss,ssr,
     2     lsp9,cp9,sshf9,ewss9,nsss9,ssr9,
     3     lsp6,cp6,sshf6,ewss6,nsss6,ssr6)
90    continue
*
* Now loop over remaining available times
      itime=3
10    itime=itime+1

!      PRINT*, itime

      if(itime.gt.maxtime) then
         write(*,*) 'NOTE: MAX. ALLOWED TIMESTEPS EXCEEDED'
         goto 300
      endif
*
      do 200 ind=1,6
        if (jhrf.eq.3) then
          if (mod(itime,4).eq.1) then
        call PBGRIB(kunit1,INBUFF,IDIMENS*8,lenght,ierr)
          else if (mod(itime,4).eq.2) then 
        call PBGRIB(kunit2,INBUFF,IDIMENS*8,lenght,ierr)
          else if (mod(itime,4).eq.3) then
        call PBGRIB(kunit3,INBUFF,IDIMENS*8,lenght,ierr)
          else 
        call PBGRIB(kunit4,INBUFF,IDIMENS*8,lenght,ierr)
          end if
        else
          if (mod(itime,2).eq.1) then
        call PBGRIB(kunit2,INBUFF,IDIMENS*8,lenght,ierr)
          else
        call PBGRIB(kunit4,INBUFF,IDIMENS*8,lenght,ierr)
          end if
        end if
        if(ierr.eq.-1) goto 300   ! EOF DETECTED
        if(ierr.lt.-1) stop 'SORRY: GRIB READ ERROR DETECTED'
        call GRIBEX (ISEC0,ISEC1,ISEC2,ZSEC2,ISEC3,ZSEC3,ISEC4,
     &               ZSEC4,IDIMENS,INBUFF,IDIMENS*8,iword,mode,ierr)
        if(ierr.ne.0) stop 'SORRY: GRIB CONVERSION ERROR DETECTED'
        ic=isec1(21)-1
        ij=isec1(10)
        im=isec1(11)
        id=isec1(12)
        ih=isec1(13)
        idel=isec1(16)
        jmd=ic*1000000+ij*10000+im*100+id
        hms=ih*10000
        jul=juldate(jmd,hms)
        jul=jul+dble(idel)/dble(24.0)
        call caldate(jul,jmd,hms)
c        idate(itime)=(jmd-ic*1000000)*100+hms/10000
        idate(itime)=jmd*100+hms/10000
c        if(ind.eq.1) write(*,'(a,i8.8)') 'DATE: ',idate(itime)
        if(ind.eq.1) write(*,'(a,i10.10)') 'DATE: ',idate(itime)
        if(mod(ih,3).ne.0) stop 'SORRY: ILLEGAL TIME DETECTED' 

        do 150 i=1,nx
          do 150 j=1,ny
            if(isec1(6).eq.142) lsp (i,j,4)=zsec4(nx*(ny-j)+i)
            if(isec1(6).eq.143) cp  (i,j,4)=zsec4(nx*(ny-j)+i)
            if(isec1(6).eq.146) sshf(i,j,4)=zsec4(nx*(ny-j)+i)
            if(isec1(6).eq.176) ssr (i,j,4)=zsec4(nx*(ny-j)+i)
            if(isec1(6).eq.180) ewss(i,j,4)=zsec4(nx*(ny-j)+i)
            if(isec1(6).eq.181) nsss(i,j,4)=zsec4(nx*(ny-j)+i)
150     continue
        if(isec1(6).eq.142) then
          write(*,'(6x,a,4x,i3)') 
     &      'FINISHED: LSP ',isec1(6)
        endif
        if(isec1(6).eq.143) then
          write(*,'(6x,a,4x,i3)') 
     &      'FINISHED: CP  ',isec1(6)
        endif
        if(isec1(6).eq.146) then
          write(*,'(6x,a,4x,i3)') 
     &      'FINISHED: SSHF',isec1(6)
        endif
        if(isec1(6).eq.176) then
          write(*,'(6x,a,4x,i3)') 
     &      'FINISHED: SSR ',isec1(6)
        endif
        if(isec1(6).eq.180) then
          write(*,'(6x,a,4x,i3)') 
     &      'FINISHED: EWSS',isec1(6)
        endif
        if(isec1(6).eq.181) then
          write(*,'(6x,a,4x,i3)') 
     &      'FINISHED: NSSS',isec1(6)
        endif
200   continue
*
* Convert fluxes to consistent 3- or 6- hourly RATEs
      call rate(itime,nx,ny,jpoly,
     1     lsp,cp,sshf,ewss,nsss,ssr,
     2     lsp9,cp9,sshf9,ewss9,nsss9,ssr9,
     3     lsp6,cp6,sshf6,ewss6,nsss6,ssr6)
*
* Check time
      if (idate(itime-1).gt.idate(itime)) then
         stop 'ERROR: TIMES FOR FLUX DATA NOT CORRECT'
      end if
*
* Interpolate deaccumulated fluxes with a cubic-polynomial solution
* or with a modified linear solution for rainfall
      call darain(lsp,lspi,nx,ny,jpoly)
      call darain(cp,cpi,nx,ny,jpoly)
      call dapoly(sshf,sshfi,nx,ny,jpoly)
      call dapoly(ssr,ssri,nx,ny,jpoly)
      call dapoly(ewss,ewssi,nx,ny,jpoly)
      call dapoly(nsss,nsssi,nx,ny,jpoly)
*
* Write out interpolated / deacc. fields at this timestep
      call writeout(itime,ISEC0,ISEC1,ISEC2,ZSEC2,ISEC3,
     1     ZSEC3,ISEC4,ZSEC4,INBUFF,iword,
     2     lspi,cpi,sshfi,ewssi,nsssi,ssri,idate,
     3     nx,ny,maxtime,idimens)
C *** Shift flux fields back one unit in time
      call daback(lsp,nx,ny,jpoly)
      call daback(cp,nx,ny,jpoly)
      call daback(sshf,nx,ny,jpoly)
      call daback(ssr,nx,ny,jpoly)
      call daback(ewss,nx,ny,jpoly)
      call daback(nsss,nx,ny,jpoly)

      goto 10
*
* CLOSING INPUT DATA FILES
*
300   continue
      call pbclose(kunit2,ierr) 
      call pbclose(kunit4,ierr) 
      if (jhrf.eq.3) then
         call pbclose(kunit1,ierr)
         call pbclose(kunit3,ierr)
      end if

      return
      end 
