#!/bin/ksh


# This is a utility script written by Leopold Haimberger for the purpose of an convenient upload
# of his ECMWF data extraction and retrieval script. The script belongs to the SW package
# ECMWFDATA V2.0!
#
# Prerequisites:
# 	       Mandatory: A valid ECWMF MS user account
# (Strongly) recommended: An installation of the ecaccess Gateway server on your local computer system 
#              so you can launch your requests and receive the data on your side of the VPN
#              protected  gateway. [In dependence of your firewall settings it might be required
#              to put the local gateway host into the DMZ (demilitarized zone) domain. Please consult
#              the ecaccess software documentation available at ECMWF for this issue].
#              In cases no gateway installation is available, you have to modify the below script
#              so it runs directly on the 'ecgate' host.
# Usage: 
# In this top-level script the environment variables
# $GATEWAY $DESTINATION  $ECUID  $ECPATH
# are user specific. YOU HAVE TO ADJUST THEM!!!
# 

[ -z "$GATEWAY" ] && GATEWAY=xxx.net
[ -z "$DESTINATION" ] && DESTINATION=xxx@xxx
[ -z "$ECUID" ] && ECUID=xxx
[ -z "$ECPATH" ] && ECPATH=xxx/

echo 'ECMWFDATA v2.0 is installed for:'
echo Gateway     $GATEWAY 
echo Destination $DESTINATION 
echo ECMWF user ID $ECUID 
echo ECMWF scratch path /home/ms/$ECPATH$ECUID

echo 'Note: These settings can be changed via environment variables'
echo '$GATEWAY $DESTINATION  $ECUID  $ECPATH'

# Depending on the retrieval methodology you want to apply (for details see ECMWFDATA_V2.0 documentation) three different
# scripts are available:
# (a) flex_ecmwf_91_finegrid_ecgate: For tailored data retrieval on grids of 0.5 degree or higher resolution performed on 'ecgate'
# (b) flex_ecmwf_91_finegrid_hpce:   For tailored data retrieval on grids of 0.5 degree or higher resolution performed on 'hpce'
# (c) flex_ecmwf_91_global_1.0_ecgate: For global data retrieval on 1.0 degree resolution performed on 'ecgate'
# Note: Any modifications other then the aforementioned env. variables have to be edited directly in these 3 retrieval scripts!


SERVER=`uname -n`
if [[ ${SERVER} != ecgate ]] ; then

echo "Software copied from server ${SERVER}" to ecgate

export ECDOMAIN=ECHOME
ksh upload_source

# automatically set gateway and destination for on demand scripts
sed "s,xxx.net,${GATEWAY}," flex_ecmwf_91_finegrid_hpce.tmp | sed "s,xxx@genericFtp,${DESTINATION}," > flex_ecmwf_91_finegrid_hpce
sed "s,xxx.net,${GATEWAY}," flex_ecmwf_91_finegrid_ecgate.tmp | sed "s,xxx@genericFtp,${DESTINATION}," > flex_ecmwf_91_finegrid_ecgate
sed "s,xxx.net,${GATEWAY}," flex_ecmwf_91_global_1.0_ecgate.tmp | sed "s,xxx@genericFtp,${DESTINATION}," > flex_ecmwf_91_global_1.0_ecgate

ecmkdir flex_extract_ecgate_V2
ecput flex_ecmwf_91_finegrid_hpce flex_extract_ecgate_V2/flex_ecmwf_91_finegrid_hpce
ecput flex_ecmwf_91_finegrid_ecgate flex_extract_ecgate_V2/flex_ecmwf_91_finegrid_ecgate
ecput flex_ecmwf_91_global_1.0_ecgate flex_extract_ecgate_V2/flex_ecmwf_91_global_1.0_ecgate

export ECDOMAIN=ECSCRATCH
ecmkdir ms_sms_output_V2
export ECDOMAIN=ECHOME

echo 'submitting example  on demand script flex_ecmwf_91_global_1.0_ecgate...'
ecjput ecgate flex_ecmwf_91_global_1.0_ecgate

else
echo "Expanding templates in local directory"

# automatically set gateway and destination for on demand scripts
sed "s,xxx.net,${GATEWAY}," flex_ecmwf_91_finegrid_hpce.tmp | sed "s,xxx@genericFtp,${DESTINATION}," > flex_ecmwf_91_finegrid_hpce
sed "s,xxx.net,${GATEWAY}," flex_ecmwf_91_finegrid_ecgate.tmp | sed "s,xxx@genericFtp,${DESTINATION}," > flex_ecmwf_91_finegrid_ecgate
sed "s,xxx.net,${GATEWAY}," flex_ecmwf_91_global_1.0_ecgate.tmp | sed "s,xxx@genericFtp,${DESTINATION}," > flex_ecmwf_91_global_1.0_ecgate

mkdir $SCRATCH/ms_sms_output_V2


echo 'submitting example  on demand script flex_ecmwf_91_global_1.0_ecgate...'
ecjput ecgate flex_ecmwf_91_global_1.0_ecgate

fi
